/* data model for this app */
.pragma library

var debug = false;
var _tart = null;
var Application = null;
var _handlers = {}; // registered handlers, by name

var Tart = {};

// globals at this point are purely the standard JS ones, plus Qt and the translate stuff

function init(__tart, app) {
    _tart = __tart;
    Application = app;

    _tart.messageYielded.connect(Tart._onMessage);

    // temporary
    Application.manualExit.connect(function () { dispatch('manualExit') });
    Application.thumbnail.connect(function () { dispatch('windowStateChanged', {state: 'thumbnail'}) });
    Application.invisible.connect(function () { dispatch('windowStateChanged', {state: 'invisible'}) });
    Application.fullscreen.connect(function () { dispatch('windowStateChanged', {state: 'fullscreen'}) });
    Application.asleep.connect(function () { dispatch('appStateChanged', {state: 'background'}) });
    Application.awake.connect(function () { dispatch('appStateChanged', {state: 'foreground'}) });
    Application.swipeDown.connect(function () { dispatch('swipeDown') });

    Application.autoExit = false;
}


var send = Tart.send = function(type, data) {
    _tart.postMessage(JSON.stringify([type, data]));
}

function register(namespace) {
    if (debug)
        print('tart.register', typeof namespace, Object.getOwnPropertyNames(namespace));

    for (var name in namespace) {
        if (/on[A-Z]/.test(name) && typeof namespace[name] == 'function') {
            _handlers[name] = namespace[name];
            if (debug)
                print('registered handler', name);
        }
        else {
            if (debug)
                print('not registering', name, 'as handler', typeof namespace[name]);
        }
    }
}


Tart._onMessage = function(msg) {
    if (debug)
        console.log('_onMessage', msg);

    var parts = JSON.parse(msg);
    var type = parts[0];
    var data = parts[1];

    dispatch(type, data);
}

function dispatch(type, data) {
    if (!type) {
        console.log('ERROR, empty type');
    } else {
        type = 'on' + type[0].toUpperCase() + type.substring(1);
        var signal = _handlers[type];
        if (!signal) {
            signal = Tart[type];
            if (!signal) {
                console.log('ERROR, signal not found', type);
                return;
            }
        }

        // console.log('signalling', typeof signal, type);
        signal(data);
    }
}


// debugging support, use with App.js() in app.py
Tart.onEvalJavascript = function(data) {
    console.log('eval this', data.text);
    var result = eval(data.text);
    console.log('.. result', result);
    Tart.send('evalResult', {value: result});
}


Tart.onContinueExit = function() {
    // everyone else is done, so let's blow this popsicle stand
    Application.quit();
}


Tart.onBackendReady = function() {
    if (debug)
        console.log('backend is ready, duh');
}


Tart.onManualExit = function() {
    if (debug)
        console.log('** onManualExit');
    Tart.send('manualExit');
}


// EOF
